#include <DualWheelTruck.h> // Подключение библиотеки <DualWheelTruck.h> для работы с моторами
#include <SmoothServo.h> // Подключение библиотеки <SmoothServo.h> для работы с сервоприводами

#define KOVSH_UP 0 // Назначение угла поворота вала сервопривода, при котором ковш поднят
#define KOVSH_DOWN 40 // Назначение угла поворота вала сервопривода, при котором ковш опущен

#define LEFT_LED 6 // Назначение пина, к которому подключен левый светодиод
#define RIGHT_LED 11 // Назначение пина, к которому подключен правый светодиод
#define BUTTON_START 5 // Назначение пина, к которому подключена тактовая кнопка запуска
#define LINE_SENSOR 13 // Назначение пина, к которому подключен датчик линии

bool robot_state=false; // Б.1 - Объявление переменной robot_state и присвоение ей значения false
bool sensor_state; // Объявление переменной sensor_state
bool button_state; // Объявление переменной button_state 

int speeds=100; // Б.1 - Объявление переменной speeds и присвоение ей значения 100
int counter; // Объявление переменной counter
int timer; // Объявление переменной timer 

void button_read(); // Объявление функции button_read()
void zapusk(); // Объявление функции zapusk()

DCMotor* motor_right; // Инициализация мотора и присвоение ему имени motor_right
DCMotor* motor_left; // Инициализация мотора и присвоение ему имени motor_left

SmoothServo* kovsh; // Инициализация сервопривода и присвоение ему имени kovsh

void setup() {
Board::vertor_PCA(); // Объявление типа используемого контроллера - vertor_PCA

motor_right = new DCMotor(1, 0, true, 5); // Настройка мотора motor_right с применением функции DCMotor(a, b, c, d): подключен к клеммнику 1, прямая полярность подключения мотора, инерционность 5
motor_left = new DCMotor(0, 0, false, 5); // Настройка мотора motor_left с применением функции DCMotor(a, b, c, d): подключен к клеммнику 0, обратная полярность подключения мотора, инерционность 5

motor_right->stop(false); // Команда на остановку вращения мотора motor_right
motor_left->stop(false); // Команда на остановку вращения мотора motor_left

kovsh = new SmoothServo(15); // Указание номера штыря, к которому подключен сервопривод kovsh

kovsh->setMinAngularSpeed(70); // Установка минимальной угловой скорости поворота вала сервопривода kovsh
kovsh->setMaxAngularSpeed(180); // Установка максимальной угловой скорости поворота вала сервопривода kovsh

kovsh->begin(30); // Команда на запуск работы с сервоприводом kovsh

kovsh->smoothRotate(KOVSH_UP); // Б.2 - Команда на плавный поворот вала серводвигателя kovsh в положение KOVSH_UP градусов

pinMode (LEFT_LED,OUTPUT); // Настройка пина LEFT_LED на вывод
pinMode (RIGHT_LED,OUTPUT); // Настройка пина RIGHT_LED на вывод
pinMode (BUTTON_START,INPUT); // Настройка пина BUTTON_START на ввод
pinMode (LINE_SENSOR,INPUT); // Настройка пина LINE_SENSOR на ввод
}

void loop() {      
  button_read(); // Б.3 - Вызов на исполнение функции button_read
  if(robot_state==true) // Б.4 - Если значение переменной robot_state равно true, то начать выполнение инструкций в скобках
  {   
  zapusk(); // Б.5 - Вызов на исполнение функции zapusk    
  while(robot_state==true) // Б.6 - Выполнение инструкций в скобках пока значение переменной robot_state равно true     
  {
     sensor_state=digitalRead(LINE_SENSOR); // Б.7 - Сохранение состояния на пине LINE_SENSOR в переменную sensor_state
     if(sensor_state==true) // Б.8 - Если значение переменной sensor_state равно true, то
     {
      motor_right->forward(speeds); // Б.9 - Команда на запуск мотора motor_right со скоростью speeds % в прямом направлении
      motor_left->forward(speeds); // Б.9 - Команда на запуск мотора motor_left со скоростью speeds % в прямом направлении
     }
     else // Иначе значение переменной равно false (под датчиком линии черный цвет)
     {
      motor_right->stop(false); // Б.10 - Команда на остановку вращения мотора motor_right
      motor_left->stop(false); // Б.10 - Команда на остановку вращения мотора motor_left
      kovsh->smoothRotate(KOVSH_UP); // Б.11 - Команда на плавный поворот вала серводвигателя kovsh в положение KOVSH_UP градусов 
      delay(400); // Б.12 - Ожидание 0,4 секунд
      motor_right->backward(speeds); // Б.13 - Команда на запуск мотора motor_right со скоростью speeds % в обратном направлении
      motor_left->backward(speeds); // Б.13 - Команда на запуск мотора motor_left со скоростью speeds % в обратном направлении
      delay(600); // Б.14 - Ожидание 0,6 секунд
      motor_right->backward(speeds); // Б.15 - Команда на запуск мотора motor_right со скоростью speeds % в обратном направлении
      motor_left->forward(speeds); // Б.15 - Команда на запуск мотора motor_left со скоростью speeds % в прямом направлении
      timer=rand()%50+51; // Б.16 - Присвоение переменной timer случайного значения от 50 до 100      
      while (timer>0) // Б.17 - Цикл While(). Выполнять до тех пор, пока истинно выражение timer>0
      {
        delay(10); // Б.27 - Ожидание 0,01 секунды       
        sensor_state=digitalRead(LINE_SENSOR); // Б.28 - Сохранение состояния на пине LINE_SENSOR в переменную sensor_state
        if(sensor_state==false)  // Б.29 - Если значение переменной button_state равно true, то
        {
          motor_right->backward(speeds); // Б.30 - Команда на запуск мотора motor_right со скоростью speeds % в обратном направлении
          motor_left->backward(speeds); // Б.30 - Команда на запуск мотора motor_left со скоростью speeds % в обратном направлении
          delay(500); // Б.31 - Ожидание 0,5 секунд
          break; // Команда на прерывание цикла разворота робота
        } 
         timer=timer-1; // Б.32 - Уменьшение значения переменной timer на 1       
      }
       motor_right->stop(false); // Б.18 - Команда на остановку вращения мотора motor_right
       motor_left->stop(false); // Б.18 - Команда на остановку вращения мотора motor_left
       kovsh->smoothRotate(KOVSH_DOWN); // Б.19 - Команда на плавный поворот вала серводвигателя kovsh в положение KOVSH_DOWN градусов  
       delay(400); // Б.20 - Ожидание 0,4 секунд   
     }
     button_state=digitalRead(BUTTON_START); // Б.21 - Сохранение состояния на пине BUTTON_START в переменную button_state
     if(button_state==0) // Б.22 - Если значение переменной button_state равно 0, то 
     {
      robot_state=false; // Б.23 - Присвоение переменной robot_state значение false 
      motor_right->stop(false); // Б.24 - Команда на остановку вращения мотора motor_right
      motor_left->stop(false); // Б.24 - Команда на остановку вращения мотора motor_left      
      kovsh->smoothRotate(KOVSH_UP); // Б.25 - Команда на плавный поворот вала серводвигателя kovsh в положение KOVSH_UP градусов
      delay(1500); // Б.26 - Ожидание 1,5 секунд
     }     
  }
  }
}

void button_read()
{
  button_state=digitalRead(BUTTON_START); // Б.3.1 - Сохранение состояния на пине BUTTON_START в переменную button_state
  if(button_state==0) // Б.3.2 - Если значение переменной button_state равно 0, то 
  {
  delay(10); // Б.3.3 - Ожидание 0,01 с
  button_state=digitalRead(BUTTON_START); // Б.3.4 - Сохранение состояния на пине BUTTON_START в переменную button_state
  if(button_state==0) // Б.3.5 - Если значение переменной button_state равно 0, то 
  {
  do 
  {
   button_state=digitalRead(BUTTON_START); // Б.3.6 - Сохранение состояния на пине BUTTON_START в переменную button_state
  }
  while(button_state==0); // Б.3.7 - Цикл do..while() выполнять до тех пор, пока значение переменной button_state равно 0
   robot_state=true; // Б.3.8 - Присвоение переменной robot_state значение true   
  }
  }
}

void zapusk()
{
  counter=0; // Б.5.1 - Присвоение переменной counter значения 0
  while(counter<3) // Б.5.2 - Инициализация цикла While()
                   // Выполнение цикла, пока значение переменной counter меньше 3                                     
  {
   digitalWrite(LEFT_LED,HIGH); // Б.5.3 - Установить на пине LEFT_LED напряжение, соответствующее уровню "HIGH"
   digitalWrite(RIGHT_LED,HIGH); // Б.5.3 - Установить на пине RIGHT_LED напряжение, соответствующее уровню "HIGH"
   delay(500); // Б.5.4 - Ожидание 0,5 секунды
   digitalWrite(LEFT_LED,LOW); // Б.5.5 - Установить на пине LEFT_LED напряжение, соответствующее уровню "LOW"
   digitalWrite(RIGHT_LED,LOW); // Б.5.5 - Установить на пине RIGHT_LED напряжение, соответствующее уровню "LOW"
   delay(500); // Б.5.6 - Ожидание 0,5 секунд
   counter=counter+1; // Б.5.7 - Увеличение значения переменной counter на 1
  }  
   kovsh->smoothRotate(KOVSH_DOWN); // Б.5.8 - Команда на плавный поворот вала серводвигателя kovsh в положение KOVSH_DOWN градусов
   delay(400); // Б.5.9 - Ожидание 0,4 с      
}
